META_EDGE = "edge"
META_ITERATION = "iteration"
META_BRACKET_ITERATION = "bracket_iteration"
META_REWRITE_PATH = "rewrite_path"
META_IS_EXTERNAL = "is_external"
META_HAS_EARLY_STOPPING = "has_early_stopping"
META_HAS_JOBS = "has_jobs"
META_HAS_SERVICES = "has_services"
META_HAS_DAGS = "has_dags"
META_HAS_MATRICES = "has_matrices"
META_HAS_SCHEDULES = "has_schedules"
META_HAS_HOOKS = "has_hooks"
META_IS_HOOK = "is_hook"
META_IS_ACTION = "is_action"
META_IS_TUNER = "is_tuner"
META_IS_PROMOTED = "is_promoted"
META_CONCURRENCY = "concurrency"
META_HAS_DOWNSTREAM_EVENTS_TRIGGER = "has_downstream_events_trigger"
META_UPLOAD_ARTIFACTS = "upload_artifacts"
META_COPY_ARTIFACTS = "copy_artifacts"
META_DESTINATION_IMAGE = "destination_image"
META_PROGRESS = "progress"
META_RECOMPILE = "recompile"
