from typing import Optional

from pydantic import StrictStr

from polyaxon.schemas.types.base import BaseTypeConfig


class V1EventType(BaseTypeConfig):
    _IDENTIFIER = "event"

    name: Optional[StrictStr]
    kind: Optional[StrictStr]
